import requests

from nbb.models import NextPass


# curl -X 'GET'
# 'https://prim.iledefrance-mobilites.fr/marketplace/stop-monitoring?MonitoringRef=STIF%3AStopArea%3ASP%3A420704%3A'
# -H 'accept: application/json'
# -H  'apikey: HjdLNrITbnGeVPpXSoPrwA0n3gQ6K02W' | jq

BASE_URL = "https://prim.iledefrance-mobilites.fr/marketplace/"
DEFAULT_API_KEY = "HjdLNrITbnGeVPpXSoPrwA0n3gQ6K02W"


def get_raw_data(STOP_ID):
    """Get raw data from API and return it as a json object."""
    response = requests.get(
        BASE_URL
        + "stop-monitoring?MonitoringRef="
        + f"STIF:StopArea:SP:{STOP_ID}:".replace(":", "%3A"),
        headers={"apikey": DEFAULT_API_KEY, "accept": "application/json"},
    )
    return response.json()


def get_next_passes(stop_id):
    """Get next passes from API and return it as a list of NextPass objects."""
    raw_data = get_raw_data(stop_id)
    next_passes = []
    monit_list = raw_data
    for f in [
        "Siri",
        "ServiceDelivery",
        "StopMonitoringDelivery",
        0,
        "MonitoredStopVisit",
    ]:
        monit_list = monit_list[f]

    for monitored in monit_list:
        next_passes.append(NextPass.from_v2(monitored))
    return next_passes



# /usr/bin/env python3
"""
CLI frontend.
"""
import argparse
import datetime

from nbb.api_call import get_next_passes
from nbb.config import get_config, get_stop_infos


def parser():
    """Initialize parser."""
    parser = argparse.ArgumentParser()
    parser.add_argument("--config", default=None)
    parser.add_argument("--raw", action="store_true")
    parser.add_argument("--simple", action="store_true")
    parser.add_argument("--compact", action="store_true")
    parser.add_argument("--verbose", action="store_true")
    parser.add_argument("stop_name", nargs="?", default=None)
    ns = parser.parse_args()
    if ns.verbose:
        print("This is nbb cli\nVerbose = ON")
    return ns


def main():
    """Execute main CLI function."""
    ns = parser()
    if ns.verbose:
        print("loading config")

    conf = get_config(ns.config)
    # Get the default stop as the first one registered:
    stop_name, stop_code, filters = get_stop_infos(conf, ns.stop_name)

    next_passes = get_next_passes(stop_code)
    # Remove past buses
    for i, n in enumerate(next_passes):
        if n.time < datetime.datetime.now().astimezone():
            n.is_valid = False
    # Filter by direction
    if direction_filter := conf["stop"]["direction_filter"].get(stop_name, None):
        for i, n in enumerate(next_passes):
            if n.destination not in direction_filter:
                n.is_valid = False

    print(
        f"Next buses at {stop_name} planned at "
        f"{datetime.datetime.now().astimezone().strftime('%H:%M')}"
    )
    ret_string = "\n".join(
        n.as_str(ns.compact, pretty=not ns.simple) for n in next_passes if n.is_valid
    )
    print(ret_string)


if __name__ == "__main__":
    main()



from dataclasses import dataclass
from datetime import datetime, timedelta

# TODO use ratp dataset
KNOWN_LINES = {"C01561": "9", "C01567": "91.06"}

NUM2EMOJI = {
    "0": "0️⃣ ",
    "1": "1️⃣ ",
    "2": "2️⃣ ",
    "3": "3️⃣ ",
    "4": "4️⃣ ",
    "5": "5️⃣ ",
    "6": "6️⃣ ",
    "7": "7️⃣ ",
    "8": "8️⃣ ",
    "9": "9️⃣ ",
    "10": "🔟 ",
}


def _get_value_id(obj, name) -> tuple:  # TODO Use namedtuple
    val = obj[name]
    if isinstance(val, list):
        val = val[0]
    val = val["value"]

    t, q, id = val.split(":")[1:-1]
    return t, q, id


@dataclass
class NextPass:
    """Description of the next pass of a bus at a stop area."""

    line_name: str
    """Name of the line."""
    destination: str
    """Destination of the bus."""
    time: datetime
    """Time of the next pass."""
    arrival_status: str
    """Arrival status of the bus. eg: onTime"""
    stop_area_name: str
    """Name of the stop area."""
    stop_area_id: int
    """ID of the stop area."""
    line_id: int
    """ID of the line."""
    is_valid: bool = True

    @classmethod
    def from_v1(cls, data):
        pass

    @classmethod
    def from_v2(cls, data: dict):
        """Create a NextPass object from the v2 data."""
        journey = data["MonitoredVehicleJourney"]
        call = journey["MonitoredCall"]

        stop_area_id = _get_value_id(data, "MonitoringRef")
        stop_area_name = call["StopPointName"][0]["value"]
        # TODO Save the mapping stop_area_id -> stop_area_name
        # Or dowload the full dataset from IDFM.
        line_id = _get_value_id(journey, "LineRef")[-1]
        line_name = KNOWN_LINES.get(line_id, "Unknown")
        return cls(
            destination=journey["DestinationName"][0]["value"],
            time=datetime.fromisoformat(call["ExpectedArrivalTime"]).astimezone(),
            arrival_status=call["ArrivalStatus"],
            stop_area_name=stop_area_name,
            stop_area_id=stop_area_id,
            line_id=line_id,
            line_name=line_name,
        )

    @property
    def delta_time(self) -> timedelta:
        """Returns the time difference between now and the next pass."""

        return self.time - datetime.now(tz=self.time.tzinfo)

    def as_str(self, compact: bool = False, pretty: bool = True) -> str:
        """Returns a pretty string representation of the next pass."""

        if compact:
            destination = "".join(
                [i for i in self.destination if i.isnumeric() or i.isupper()]
            )
            time_str = self.time.astimezone().strftime("%H:%M")
        else:
            time_str = (
                f"{self.delta_time.seconds // 60:>2} ({self.time.strftime('%H:%M')})"
            )
            destination = self.destination

        if pretty:
            bus_pretty_name = "".join([NUM2EMOJI.get(i, "") for i in self.line_name])
            return f"⏰ {time_str} {bus_pretty_name: <10} 🚍▶ {destination}"

        return f"{time_str} {self.line_name} [{destination}]"

    def __le__(self, other):
        return self.time <= other.time



"""Handles API calls to IDFM servers."""


from nbb.models import NextPass

BASE_URLv1 = ""

BASE_URLv2 = ""


def get_next_buses(request_data) -> list[NextPass]:
    """Parse the Request and Create the NextPass List"""
    list_monitored = request_data["Siri"]["ServiceDelivery"]["StopMonitoringDelivery"][
        0
    ]["MonitoredStopVisit"]

    nextpass_list = [None] * len(list_monitored)
    for i, obj in enumerate(list_monitored):
        nextpass_list[i] = NextPass.from_v2(obj)

    return nextpass_list



import os

try:
    import tomllib as toml
except ImportError:
    import tomli as toml

from importlib.resources import files

import unicodedata


def find_config(input_cli):
    """Find the config file. the order of precedence is:
    1. cli argument
    2. environment variable
    3. ~/.config/nbb/nbb_conf.toml
    4. /etc/nbb/nbb_conf.toml
    5. default config file in the same directory as this file
    """
    if input_cli is not None:
        config_file = input_cli
    elif "NBB_CONF" in os.environ:
        config_file = os.environ["NBB_CONF"]
    elif os.path.exists(os.path.expanduser("~/.config/nbb/nbb_conf.toml")):
        config_file = os.path.expanduser("~/.config/nbb/nbb_conf.toml")
    elif os.path.exists("/etc/nbb/nbb_conf.toml"):
        config_file = "/etc/nbb/nbb_conf.toml"
    else:
        config_file = files("nbb").joinpath("nbb_conf.toml")
    return config_file


def get_config(config_file=None):
    """Load config file as nested dictionary."""
    if config_file is None:
        config_file = find_config(config_file)
    with open(config_file, "rb") as f:
        try:
            conf = toml.load(f)
        except FileNotFoundError:
            print(f"{config_file} specified but not found.")
            exit(1)
        except toml.TOMLDecodeError as e:
            print(e)
            exit(1)
    return conf


def normalize(config):
    """Normalize all the aliases."""

    def string_norm(string):
        return unicodedata.normalize("NFKD", string).lower()

    alias_table = {}
    for stop_name, aliases in config["stop"]["aliases"].items():
        for alias_name in aliases:
            alias_table[string_norm(alias_name)] = stop_name

        alias_table[string_norm(stop_name)] = stop_name
    return alias_table


def get_stop_infos(config, input_name):
    """Get the line code and stop area code."""
    alias_table = normalize(config)

    if input_name is not None:
        try:
            stop_name = alias_table[input_name]
        except KeyError:
            print("Provided stop name could not be map to a registered stop name.")
            exit(1)
    else:
        stop_name = list(config["stop"]["places"])[0]
    stop_code = config["stop"]["places"][stop_name]
    filters = config["stop"]["direction_filter"].get(stop_name, [])
    return stop_name, stop_code, filters



try:
    # -- Distribution mode --
    # import from _version.py generated by setuptools_scm during release
    from ._version import version as __version__
except ImportError:
    # -- Source mode --
    # use setuptools_scm to get the current version from src using git
    from setuptools_scm import get_version as _gv
    from os import path as _path

    __version__ = _gv(_path.join(_path.dirname(__file__), _path.pardir))
